program Inter_demo;
{$APPTYPE CONSOLE}
uses
  windows,
  classes,
  poa,
  poa_int,
  poa_impl,
  SysUtils,
  orb_int,
  orb,
  code_int,
  env_int,
  Intercept_int,
  Intercept,
  test_int in 'test_int.pas',
  test in 'test.pas';

type
  TTestCon = class(TConnInterceptor)
  protected
    function client_connect(const addr: string): TStatus; override;
    function client_disconnect(const addr: string): TStatus; override;
  end;

  TTestServ = class(TServerInterceptor)
  protected
    function input_message(const buf: IBuffer; const env: IEnvironment): TStatus; override;
    function initialize_request(const req: ILWServerRequest; const env: IEnvironment): TStatus; override;
    function after_unmarshal(const req: ILWServerRequest; const env: IEnvironment): TStatus; override;
    function before_marshal(const req: ILWServerRequest; const env: IEnvironment): TStatus; override;
    function finish_request(const req: ILWServerRequest; const env: IEnvironment): TStatus; override;
    function output_message(const buf: IBuffer; const env: IEnvironment): TStatus; override;
  end;

  TTestClient = class(TClientInterceptor)
  protected
    function initialize_request(const req: ILWRequest; const env: IEnvironment): TStatus; override;
    function after_marshal(const req: ILWRequest; const env: IEnvironment): TStatus; override;
    function output_message(const buf: IBuffer; const env: IEnvironment): TStatus; override;
    function input_message(const buf: IBuffer; const env: IEnvironment): TStatus; override;
    function before_unmarshal(const req: ILWRequest; const env: IEnvironment): TStatus; override;
    function finish_request(const req: ILWRequest; const env: IEnvironment): TStatus; override;
  end;
  
  TMyObj = class(TMyTest_serv)
  protected
    procedure Foo(var str: string); override;
  end;

procedure mainproc;
var
  dorb : IORB;
  obj: IORBObject;
  rootPOA: IPOA;
  mgr: IPOAManager;
  connIntr: IConnInterceptor;
  servIntr: IServerInterceptor;
  clientIntr: IClientInterceptor;
  bank: TMyObj;
  fl : TStrings;
  ref: string;
  myintf: IMyTest;
begin
  connIntr := TTestCon.Create;
  connIntr.activate(0);
  servIntr := TTestServ.Create;
  servIntr.activate(0);
  clientIntr := TTestClient.Create;
  clientIntr.activate(0);
  dorb := ORB_Init(nil);

  if (ParamCount = 1) and (lowercase(ParamStr(1)) = 's') then
    begin
      //Obtain a reference to the RootPOA and its Manager
      obj := dorb.resolve_initial_reference('RootPOA');
      rootPOA := TPOA_impl._narrow(obj);
      mgr := rootPOA.the_POAManager;

      bank := TMyObj.Create;

      rootPoa.activate_object(bank);

      fl := TStringList.Create;
      fl.Text := dorb.object_to_string(bank.this);
      fl.SaveToFile('server.objid');
      fl.free;
      mgr.activate;
      dorb.run;
      exit;
    end
  else if (ParamCount = 1) and (lowercase(ParamStr(1)) = 'c') then
    begin
      fl := TStringList.Create;
      fl.LoadFromFile('server.objid');
      ref := fl.Text;
      fl.free;
    end
  else if (ParamCount >= 1)  then
    begin
      writeln('usage: '+ExtractFileName(ParamStr(0))+' s|c');
      exit;
    end;
  obj := dorb.string_to_object(ref);
  myintf := TMyTest._narrow(obj);
  ref := 'Client';
  myintf.Foo(ref);
  WriteLn(ref);
end;

function ThreadID(): string;
begin
  result := '. ' + IntToStr(GetCurrentThreadID());
end;

{ TTestCon }

function TTestCon.client_connect(const addr: string): TStatus;
begin
  WriteLn('server: connect from:' + addr);
  result := INVOKE_CONTINUE;
end;

function TTestCon.client_disconnect(const addr: string): TStatus;
begin
  WriteLn('server: disconnect from:'+addr);
  result := INVOKE_CONTINUE;
end;

{ TMyObj }

procedure TMyObj.Foo(var str: string);
begin
  WriteLn(str);
  str := 'Server';
end;

{ TTestServ }

function TTestServ.after_unmarshal(const req: ILWServerRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('server: after unmarshal for: ' + req.operation+ThreadID);
  result := INVOKE_CONTINUE;
end;

function TTestServ.before_marshal(const req: ILWServerRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('server: before marshal for: ' + req.operation+ThreadID);
  result := INVOKE_CONTINUE;
end;

function TTestServ.finish_request(const req: ILWServerRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('server: finish request for: ' + req.operation+ThreadID);
  result := INVOKE_CONTINUE;
end;

function TTestServ.initialize_request(const req: ILWServerRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('server: init request for: ' + req.operation+ThreadID);
  result := INVOKE_CONTINUE;
end;

function TTestServ.input_message(const buf: IBuffer;
  const env: IEnvironment): TStatus;
var
  i: integer;
begin
  WriteLn('server: input message(' + IntToStr(buf.WPosition - buf.RPosition) + ' bytes)'+ThreadID);
  // simple decryption
  for i := buf.RPosition to buf.WPosition - 1 do
    PByteArray(buf.buffer)^[i] := PByteArray(buf.buffer)^[i] xor $37;
  result := INVOKE_CONTINUE;
end;

function TTestServ.output_message(const buf: IBuffer;
  const env: IEnvironment): TStatus;
var
  i: integer;
begin
  WriteLn('server: output message (' + IntToStr(buf.WPosition - buf.RPosition) + ' bytes)'+ThreadID);
  // simple encryption
  for i := buf.RPosition to buf.WPosition - 1 do
    PByteArray(buf.buffer)^[i] := PByteArray(buf.buffer)^[i] xor $37;
  result := INVOKE_CONTINUE;
end;

{ TTestClient }

function TTestClient.after_marshal(const req: ILWRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('client: after marshal for: ' + req.operation);
  result := INVOKE_CONTINUE;
end;

function TTestClient.before_unmarshal(const req: ILWRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('client: before unmarshal for: ' + req.operation);
  result := INVOKE_CONTINUE;
end;

function TTestClient.finish_request(const req: ILWRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('client: finish request for: ' + req.operation);
  result := INVOKE_CONTINUE;

end;

function TTestClient.initialize_request(const req: ILWRequest;
  const env: IEnvironment): TStatus;
begin
  WriteLn('client: init request for: ' + req.operation);
  result := INVOKE_CONTINUE;
end;

function TTestClient.input_message(const buf: IBuffer;
  const env: IEnvironment): TStatus;
var
  i: integer;
begin
  WriteLn('client: input message (' + IntToStr(buf.WPosition - buf.RPosition) + ' bytes)');
  // simple decryption
  for i := buf.RPosition to buf.WPosition - 1 do
    PByteArray(buf.buffer)^[i] := PByteArray(buf.buffer)^[i] xor $37;
  result := INVOKE_CONTINUE;
end;

function TTestClient.output_message(const buf: IBuffer;
  const env: IEnvironment): TStatus;
var
  i: integer;
begin
  WriteLn('client: output message (' + IntToStr(buf.WPosition - buf.RPosition) + ' bytes)');
  // simple encryption
  for i := buf.RPosition to buf.WPosition - 1 do
    PByteArray(buf.buffer)^[i] := PByteArray(buf.buffer)^[i] xor $37;
  result := INVOKE_CONTINUE;
end;

begin
  // Insert user code here
  mainproc;
end.