// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
unit codeset;

interface

uses
  windows,orbcomp,Types,classes,decoder,orb_int,orb,comp_int,code_int;

type


  TCodesetComponent = class(TORBComponent,IORBComponent,ICodesetComponent)
  private
    FNativeCodeset : CodesetID;
    FNativeWCodeset : CodesetID;
    FCodesets : TList;
    FWCodesets : TList;
  protected
    procedure encode(const enc: IEncoder); override;
  public
    constructor Create(ncs,nwcs: CodesetID; css,wcss: TList);
    destructor Destroy(); override;
    procedure print(s: TStream); override;
    function selected_cs(): CodesetID;
    function selected_wcs(): CodesetID;
  end;

  TCodesetComponentDecoder = class(TORBComponentDecoder,IORBComponentDecoder)
  private
    FCSets,FWCSets : TList;
  public
    constructor Create(id: ComponentID);
    destructor Destroy(); override;
    function decode(dec: IDecoder; cid: ComponentID; len: _ulong): IORBComponent; override;
  end;

  TCodesetConverter = class(TORBObject,ICodesetConverter)

  end;

implementation

uses
  pstream,utils,codeset_ref;


{**********************************************************************
 TCodesetComponent
**********************************************************************}
constructor TCodesetComponent.Create(ncs,nwcs: CodesetID; css,wcss: TList);
var
  i : integer;
begin
  inherited Create();
  set_id(TAG_CODE_SETS);
  FCodesets := TList.Create;
  if css <> nil then
    for i := 0 to Pred(css.Count) do
      FCodesets.add(css[i]);
  FWCodesets := TList.Create;
  if wcss <> nil then
    for i := 0 to Pred(wcss.Count) do
      FwCodesets.add(wcss[i]);
  FNativeCodeset := ncs;
  FNativeWCodeset := nwcs;
end;

destructor TCodesetComponent.Destroy();
begin
  FCodesets.Free;
  FWCodesets.Free;
  inherited Destroy();
end;

procedure TCodesetComponent.print(s: TStream);
var
  ps : TPrintStream;
  normal,wide : PCodesetInfo;
begin
  ps := TPrintStream(s);
  ps.writeln('Native Codesets:');
  normal := find_codeset(FNativeCodeset);
  wide := find_codeset(FNativeWCodeset);
  if normal <> nil then
    ps.writeln('               normal:'+normal^.desc);
  if wide <> nil then
    ps.writeln('                wide:'+wide^.desc);
end;

function TCodesetComponent.selected_cs(): CodesetID;
begin
  // must be rewritten
  result := TCodesets[0].id;
end;

function TCodesetComponent.selected_wcs(): CodesetID;
begin
  // must be rewritten
  result := TCodesets[4].id;
end;

procedure TCodesetComponent.encode(const enc: IEncoder);
var
  i : integer;
begin
  enc.put_ulong(FNativeCodeset);
  enc.seq_begin(FCodesets.Count);
  for i := 0 to Pred(FCodesets.Count) do
    enc.put_ulong(_ulong(FCodesets[i]));
  enc.seq_end;
  enc.put_ulong(FNativeWCodeset);
  enc.seq_begin(FWCodesets.Count);
  for i := 0 to Pred(FWCodesets.Count) do
    enc.put_ulong(_ulong(FWCodesets[i]));
  enc.seq_end;
end;

{**********************************************************************
 TCodesetComponentDecoder
**********************************************************************}
constructor TCodesetComponentDecoder.Create(id: ComponentID);
begin
  inherited Create(id);
  FCSets := TList.Create;
  FWCSets := TList.Create;
end;

destructor TCodesetComponentDecoder.Destroy();
begin
  FCSets.Free;
  FWCSets.Free;
  inherited Destroy();
end;

function TCodesetComponentDecoder.decode(dec: IDecoder; cid: ComponentID;
  len: _ulong): IORBComponent;
var
  native_cset,native_wcset : CodesetID;
  cnt,cs : _ulong;
begin
  result := nil;
  FCSets.Clear;
  FWCSets.Clear;
  if not dec.get_ulong(native_cset) then exit;
  if not dec.seq_begin(cnt) then exit;
  while cnt > 0 do
    begin
      if not dec.get_ulong(cs) then exit;
      FCSets.Add(Pointer(cs));
      cnt := cnt - 1;
    end;
  if not dec.seq_end() then exit;
  if not dec.get_ulong(native_wcset) then exit;
  if not dec.seq_begin(cnt) then exit;
  while cnt > 0 do
    begin
      if not dec.get_ulong(cs) then exit;
      FWCSets.Add(Pointer(cs));
      cnt := cnt - 1;
    end;
  if not dec.seq_end() then exit;
  result := TCodesetComponent.Create(native_cset,native_wcset,FCSets,FWCSets);
end;

end.
