// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
unit std_seq;

interface

uses
  stdstat,code_int,types;

type

  TStat_seq = class(TAbstractStat)
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_octet_seq = class(TStat_seq,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
  end;

  TStat_string_seq = class(TStat_seq,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
  end;

  TStat_ulong_seq = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_long_seq = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

var
  stat_seq_octet : IStaticTypeInfo;
  stat_seq_string : IStaticTypeInfo;
  stat_seq_ulong : IStaticTypeInfo;
  stat_seq_long : IStaticTypeInfo;

implementation


uses
  any,sysutils,tcode;

//*********************************************************************
//  TStat_seq
//*********************************************************************
procedure TStat_seq._create(var addr: Pointer);
begin
  addr := allocmem(sizeOf(_ulong));
end;

procedure TStat_seq._free(var addr: Pointer);
begin
  freemem(addr);
end;

//*********************************************************************
//  TStat_octet_seq
//*********************************************************************
function TStat_octet_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(OctetSeq(addr^),len);
  if not dec.get_octets(Pointer(addr^)^,len) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_octet_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
begin
  len := length(OctetSeq(addr^));
  enc.seq_begin(len);
  enc.put_octets(Pointer(addr^)^,len);
  enc.seq_end;
end;

function TStat_octet_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_octet,0);
  result := FTC;
end;

//*********************************************************************
//  TStat_string_seq
//*********************************************************************
function TStat_string_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(StringSeq(addr^),len);
  for i := 0 to Pred(len) do
    if not dec.get_string(StringSeq(addr^)[i]) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_string_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
  i : integer;
begin
  len := length(StringSeq(addr^));
  enc.seq_begin(len);
  for i := 0 to Pred(len) do
    enc.put_string(StringSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_string_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_octet,0);
  result := FTC;
end;

//*********************************************************************
//  TStat_long_seq
//*********************************************************************
procedure TStat_long_seq._assign(dst, src: Pointer);
begin
  LongSeq(dst^) := System.copy(LongSeq(src^),0,length(LongSeq(src^)));
end;

procedure TStat_long_seq._create(var addr: Pointer);
begin
  LongSeq(addr^) := nil;
end;

procedure TStat_long_seq._free(var addr: Pointer);
begin
  LongSeq(addr^) := nil;
end;

function TStat_long_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  l : _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(l) then exit;
  setLength(LongSeq(addr^),l);
  for i := 0 to pred(l) do
    if not dec.get_long(LongSeq(addr^)[i]) then exit;
  if not dec.seq_end then exit;
  result := true;
end;

procedure TStat_long_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  i : integer;
  l : _ulong;
begin
  l := length(LongSeq(addr^));
  enc.seq_begin(l);
  for i := 0 to Pred(l) do
    enc.put_long(LongSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_long_seq.typecode: ITypeCode;
begin
  result := nil;
  assert(false,'not implemented');
end;

//*********************************************************************
//  TStat_ulong_seq
//*********************************************************************
procedure TStat_ulong_seq._assign(dst, src: Pointer);
begin
  ULongSeq(dst^) := System.copy(ULongSeq(src^),0,length(ULongSeq(src^)));
end;

procedure TStat_ulong_seq._create(var addr: Pointer);
begin
  ULongSeq(addr^) := nil;
end;

procedure TStat_ulong_seq._free(var addr: Pointer);
begin
  ULongSeq(addr^) := nil;
end;

function TStat_ulong_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  l : _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(l) then exit;
  setLength(ULongSeq(addr^),l);
  for i := 0 to pred(l) do
    if not dec.get_ulong(ULongSeq(addr^)[i]) then exit;
  if not dec.seq_end then exit;
  result := true;
end;

procedure TStat_ulong_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  i : integer;
  l : _ulong;
begin
  l := length(ULongSeq(addr^));
  enc.seq_begin(l);
  for i := 0 to Pred(l) do
    enc.put_ulong(ULongSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_ulong_seq.typecode: ITypeCode;
begin
  result := nil;
  assert(false,'not implemented');
end;

initialization
  stat_seq_octet := TStat_octet_seq.Create();
  stat_seq_string := TStat_string_seq.Create();
  stat_seq_ulong := TStat_ulong_seq.Create;
  stat_seq_long := TStat_long_seq.Create;
end.
