// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
unit stdstat;

interface

uses
  types,code_int,sysutils;

type

  TAbstractStat = class(TInterfacedObject)
  protected
    procedure _assign(dst,src: Pointer); virtual;
    function copy(src: Pointer): Pointer;
    function typecode(): ITypeCode; virtual;
    procedure _create(var val: Pointer); virtual;
  end;

  TStatULong = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatLong = class(TStatUlong,IStaticTypeInfo)
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatFloat = class(TStatUlong,IStaticTypeInfo)
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatDouble = class(TStatUlong,IStaticTypeInfo)
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatBoolean = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatObject = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatAny = class(TAbstractStat,IStaticTypeInfo)
  protected
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatString = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatUShort = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatShort = class(TAbstractStat,IStaticTypeInfo)
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStatChar = class(TAbstractStat,IStaticTypeInfo)
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
    procedure _assign(dst,src: Pointer); override;
  end;

  TStatOctet = class(TAbstractStat,IStaticTypeInfo)
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
    procedure _assign(dst,src: Pointer); override;
  end;

  TStatTypeCode = class(TAbstractStat,IStaticTypeInfo)
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
    procedure _assign(dst,src: Pointer); override;
  end;

  TStatPrincipal = class(TAbstractStat,IStaticTypeInfo)
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
    procedure _assign(dst,src: Pointer); override;
  end;

var
  stat_long : IStaticTypeInfo;
  stat_ulong : IStaticTypeInfo;
  stat_float : IStaticTypeInfo;
  stat_double : IStaticTypeInfo;
  stat_Object : IStaticTypeInfo;
  stat_Any : IStaticTypeInfo;
  stat_String : IStaticTypeInfo;
  stat_Boolean : IStaticTypeInfo;
  stat_ushort : IStaticTypeInfo;
  stat_short : IStaticTypeInfo;
  stat_char : IStaticTypeInfo;
  stat_octet : IStaticTypeInfo;
  stat_typecode : IStaticTypeInfo;
  stat_principal : IStaticTypeInfo;

implementation


uses
  ior,iior_int,orb,orb_int,tcode,any,transp;

//*********************************************************************
// TAbstractStat
//*********************************************************************
procedure TAbstractStat._create(var val: Pointer);
begin
  assert(false,'not implemented');
end;

procedure TAbstractStat._assign(dst,src: Pointer);
begin
  assert(false,'not implemented');
end;

function TAbstractStat.copy(src: Pointer): Pointer;
begin
  if result <> nil then
    _assign(result,src)
  else

  _create(result);
  _assign(result,src);
end;

function TAbstractStat.typecode(): ITypeCode;
begin
  assert(false,'not implemented');
  result := nil;
end;


//**************************************************************************
// TStatULong
//**************************************************************************
procedure TStatULong._create(var addr: Pointer);
begin
  addr := AllocMem(sizeof(_ulong))
end;

procedure TStatULong._free(var addr: Pointer);
begin
  FreeMem(addr);
end;

function TStatULong.demarshal(dec: IDecoder; const addr : Pointer): Boolean;
begin
  result := dec.get_ulong(_ulong(addr^));
end;

procedure TStatULong.marshal(enc: IEncoder; const addr : Pointer);
begin
  enc.put_ulong(_ulong(addr^));
end;

function TStatULong.typecode(): ITypeCode;
begin
  result := _tc_ulong;
end;

procedure TStatULong._assign(dst, src: Pointer);
begin
  _ulong(dst^) := _ulong(src^)
end;


//**************************************************************************
// TStatULong
//**************************************************************************
procedure TStatLong._create(var addr: Pointer);
begin
  addr := AllocMem(sizeof(_ulong))
end;

procedure TStatLong._free(var addr: Pointer);
begin
  FreeMem(addr);
end;

function TStatLong.demarshal(dec: IDecoder; const addr : Pointer): Boolean;
begin
  result := dec.get_ulong(_ulong(addr^));
end;

procedure TStatLong.marshal(enc: IEncoder; const addr : Pointer);
begin
  enc.put_ulong(_ulong(addr^));
end;

function TStatLong.typecode(): ITypeCode;
begin
  result := _tc_long;
end;

//**************************************************************************
// TStatFloat
//**************************************************************************
procedure TStatFloat._create(var addr: Pointer);
begin
  addr := AllocMem(sizeof(float))
end;

procedure TStatFloat._free(var addr: Pointer);
begin
  FreeMem(addr);
end;

function TStatFloat.demarshal(dec: IDecoder; const addr : Pointer): Boolean;
begin
  result := dec.get_float(float(addr^));
end;

procedure TStatFloat.marshal(enc: IEncoder; const addr : Pointer);
begin
  enc.put_float(float(addr^));
end;

function TStatFloat.typecode(): ITypeCode;
begin
  result := _tc_float;
end;

//**************************************************************************
// TStatDouble
//**************************************************************************
procedure TStatDouble._create(var addr: Pointer);
begin
  addr := AllocMem(sizeof(double))
end;

procedure TStatDouble._free(var addr: Pointer);
begin
  FreeMem(addr);
end;

function TStatDouble.demarshal(dec: IDecoder; const addr : Pointer): Boolean;
begin
  result := dec.get_double(double(addr^));
end;

procedure TStatDouble.marshal(enc: IEncoder; const addr : Pointer);
begin
  enc.put_double(double(addr^));
end;

function TStatDouble.typecode(): ITypeCode;
begin
  result := _tc_double;
end;

//**************************************************************************
//  TStatObject
//**************************************************************************
procedure TStatObject._create(var addr: Pointer);
begin
  IORBObject(Pointer(addr^)) := IORBObject(TORBObject.Create);
end;

procedure TStatObject._assign(dst,src: Pointer);
begin
  IORBObject(dst^) := IORBObject(src^)
end;

procedure TStatObject._free(var addr: Pointer);
begin
  IORBObject(Pointer(addr^)) := nil;
end;

function TStatObject.demarshal(dec: IDecoder; const addr : Pointer): Boolean;
var
  ior : IIOR;
  obj : IORBObject;
begin
  result := false;
  ior := TIOR.Create();
  if not ior.decode(dec) then exit;
  obj := orb.ORB_Instance.ior_to_object(ior);
  IORBObject(addr^) := obj;
  result := true;
end;

procedure TStatObject.marshal(enc: IEncoder; const addr : Pointer);
var
  obj : IORBObject;
  ior : IIOR;
begin
  obj := IORBObject(addr^);
  if obj = nil then
    ior := TIOR.Create()
  else
    ior := obj.ior;
  ior.encode(enc);
end;

function TStatObject.typecode(): ITypeCode;
begin
  result := nil;
  assert(false,'not implemented');
end;

//**************************************************************************
//  TStatAny
//**************************************************************************
procedure TStatAny._create(var addr: Pointer);
begin
  IAny(addr^) := TAny.Create();
end;

procedure TStatAny._free(var addr: Pointer);
begin
  IAny(addr^) := nil;
end;

function TStatAny.demarshal(dec: IDecoder; const addr : Pointer): Boolean;
var
  a : IAny;
begin
  if Pointer(addr^) = nil then
    IAny(addr^) := TAny.Create();
  a := IAny(addr^);
  result := dec.get_any(a);
end;

procedure TStatAny.marshal(enc: IEncoder; const addr : Pointer);
var
  a : IAny;
begin
  if Pointer(addr^) = nil then
    IAny(addr^) := TAny.Create();
  a := IAny(addr^);
  enc.put_any(a);
end;

function TStatAny.typecode(): ITypeCode;
begin
  result := _tc_any
end;

procedure TStatAny._assign(dst,src: Pointer);
begin
  IAny(dst^) := TAny.Create();
  //IAny(src^).copy_any(IAny(dst^));
  IAny(dst^).copy(IAny(src^));
end;


//**********************************************************************
// TStatString
//**********************************************************************
procedure TStatString._create(var addr: Pointer);
begin
  String(addr^) := '';
end;

procedure TStatString._free(var addr: Pointer);
begin
  //Dispose(PString(addr));
  String(addr^) := '';
end;

procedure TStatString._assign(dst, src: Pointer);
begin
  string(dst^) := string(src^);
end;


function TStatString.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := dec.get_string(PString(addr)^);
end;

procedure TStatString.marshal(enc: IEncoder; const addr: Pointer);
begin
  enc.put_string(PString(addr)^);
end;

function TStatString.typecode(): ITypeCode;
begin
  result := _tc_string;
end;


//***********************************************************************
// TStatBoolean
//***********************************************************************
procedure TStatBoolean._create(var addr: Pointer);
begin
  addr := allocmem(1);
end;

procedure TStatBoolean._free(var addr: Pointer);
begin
  freemem(addr)
end;

procedure TStatBoolean._assign(dst, src: Pointer);
begin
  Boolean(dst^) := Boolean(src^);
end;

function TStatBoolean.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := dec.get_octet(Octet(addr^));
end;

procedure TStatBoolean.marshal(enc: IEncoder; const addr: Pointer);
begin
  enc.put_octet(Octet(addr^));
end;

function TStatBoolean.typecode: ITypeCode;
begin
  result := _tc_boolean;
end;

//*********************************************************************
//  TStatUShort
//*********************************************************************
procedure TStatUShort._create(var addr: Pointer);
begin
  addr := AllocMem(2);
end;

procedure TStatUShort._free(var addr: Pointer);
begin
  freemem(addr);
end;

function TStatUShort.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := dec.get_ushort(_ushort(addr^));
end;

procedure TStatUShort.marshal(enc: IEncoder; const addr: Pointer);
begin
  enc.put_short(_ushort(addr^));
end;

function TStatUShort.typecode: ITypeCode;
begin
  result := _tc_ushort;
end;

procedure TStatUShort._assign(dst, src: Pointer);
begin
  _ushort(dst^) := _ushort(src^)
end;


//*********************************************************************
//  TStatShort
//*********************************************************************
procedure TStatShort._create(var addr: Pointer);
begin
  addr := AllocMem(2);
end;

procedure TStatShort._free(var addr: Pointer);
begin
  freemem(addr);
end;

function TStatShort.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := dec.get_ushort(_ushort(addr^));
end;

procedure TStatShort.marshal(enc: IEncoder; const addr: Pointer);
begin
  enc.put_short(_ushort(addr^));
end;

function TStatShort.typecode: ITypeCode;
begin
  result := _tc_short;
end;

procedure TStatShort._assign(dst, src: Pointer);
begin
  _short(dst^) := _short(src^)
end;

//*********************************************************************
//  TStatChar
//*********************************************************************
procedure TStatChar._create(var addr: Pointer);
begin
  addr := AllocMem(1);
end;

procedure TStatChar._free(var addr: Pointer);
begin
  freemem(addr);
end;

function TStatChar.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := dec.get_octet(char(addr^));
end;

procedure TStatChar.marshal(enc: IEncoder; const addr: Pointer);
begin
  enc.put_octet(char(addr^))
end;

function TStatChar.typecode: ITypeCode;
begin
  result := _tc_char;
end;

procedure TStatChar._assign(dst, src: Pointer);
begin
  Char(dst^) := Char(src^);
end;

//*********************************************************************
//  TStatOctet
//*********************************************************************
procedure TStatOctet._create(var addr: Pointer);
begin
  addr := AllocMem(1);
end;

procedure TStatOctet._free(var addr: Pointer);
begin
  freemem(addr);
end;

function TStatOctet.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := dec.get_octet(Octet(addr^));
end;

procedure TStatOctet.marshal(enc: IEncoder; const addr: Pointer);
begin
  enc.put_octet(Octet(addr^))
end;

function TStatOctet.typecode: ITypeCode;
begin
  result := _tc_octet;
end;

procedure TStatOctet._assign(dst, src: Pointer);
begin
  Octet(dst^) := Octet(src^);
end;

//*********************************************************************
//  TStatTypeCode
//*********************************************************************
procedure TStatTypeCode._create(var addr: Pointer);
begin
  ITypeCode(addr^) := TTypeCode.Create();
end;

procedure TStatTypeCode._free(var addr: Pointer);
begin
  ITypeCode(addr^) := nil;
end;

function TStatTypeCode.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := ITypeCode(addr^).decode(dec);
end;

procedure TStatTypeCode.marshal(enc: IEncoder; const addr: Pointer);
begin
  ITypeCode(addr^).encode(enc);
end;

function TStatTypeCode.typecode: ITypeCode;
begin
  result := _tc_TypeCode;
end;

procedure TStatTypeCode._assign(dst, src: Pointer);
begin
  ITypeCode(dst^) := ITypeCode(src^);
end;

//*********************************************************************
//  TStatPrincipal
//*********************************************************************
procedure TStatPrincipal._create(var addr: Pointer);
begin
  IPrincipal(addr^) := TPrincipal.Create();
end;

procedure TStatPrincipal._free(var addr: Pointer);
begin
  IPrincipal(addr^) := nil;
end;

function TStatPrincipal.demarshal(dec: IDecoder; const addr: Pointer): Boolean;
begin
  result := IPrincipal(addr^).decode(dec);
end;

procedure TStatPrincipal.marshal(enc: IEncoder; const addr: Pointer);
begin
  IPrincipal(addr^).encode(enc);
end;

function TStatPrincipal.typecode: ITypeCode;
begin
  result := _tc_Principal;
end;

procedure TStatPrincipal._assign(dst, src: Pointer);
begin
  IPrincipal(dst^) := IPrincipal(src^);
end;

initialization
  stat_ulong := TStatULong.Create;
  stat_char := TStatChar.Create;
  stat_octet := TStatOctet.Create;
  stat_long := TStatLong.Create;
  stat_float := TStatFloat.Create;
  stat_double := TStatDouble.Create;
  stat_Object := TStatObject.Create;
  stat_Any := TStatAny.Create;
  stat_string := TStatString.Create;
  stat_boolean := TStatBoolean.Create;
  stat_ushort := TStatUShort.Create;
  stat_short := TStatShort.Create;
  stat_typecode := TStatTypeCode.Create;
  stat_principal := TStatTypeCode.Create;
end.
